﻿/**

  The Initial Developer of the Original Code is
  Matthieu  - http://www.actionscript-facile.com
  Portions created by the Initial Developer are Copyright (C) 2010
  the Initial Developer. All Rights Reserved.

  Contributor(s) :

 */
package {
	import org.osmf.containers.MediaContainer;
	import org.osmf.media.MediaPlayer;
	import org.osmf.elements.SerialElement;
	import org.osmf.net.StreamType;
	import org.osmf.net.StreamingURLResource;
	import org.osmf.media.DefaultMediaFactory;
	import org.osmf.media.MediaElement;
	import org.osmf.layout.ScaleMode;
	import org.osmf.layout.LayoutMetadata;
	import org.osmf.utils.OSMFSettings;
	import org.osmf.events.TimeEvent;
	import org.osmf.media.URLResource;

	import com.actionscriptfacile.ui.text.UITextField;
	import com.as3facile.DefaultButton;

	import flash.events.MouseEvent;
	
	import com.demonsters.debugger.MonsterDebugger;
	
	import flash.events.Event;
	import flash.display.Sprite;
	
		
	[SWF(backgroundColor="0x000000")] // definit la couleur de fond du swf
	
	
	public class Main extends Sprite
	{		
		private static const MAIL_SWF : String = "EmailFormation.swf";// l'application swf appelée par le player
		
		private static const VIDEO_STREAMING:String = "rtmp://s9teld733sjgz.cloudfront.net/cfx/st/mp4:fichiers/tony-robbins-upw";// vidéo en streaming

		public static const INTRO_VIDEO:String = "http://chilp.it/e33494";// la vidéo d'intro
		
		 
		private var oInfosDuration : UITextField;
		
		private var oMediasList : DefaultMediaFactory;// contient les vidéos de la playlist du player vidéo
		private var oMediaPlayer : MediaPlayer;// le player vidéo
		private var oMediasContainer : MediaContainer;// contient les différents éléments du player vidéo
		
		
		private var oTrackVideo:Sprite;// barre d'indication de la vidéo
		private var oPisteVideo:Sprite;// barre avance rapide de la vidéo
		private var oProgressPlayVideo:Sprite;// barre de progression de la lecture de la vidéo
		
		
		/**
		 * Constructeur.
		 * 
		 * Compiler avec le SDK Flex 4.6
		 * 
		 * -swf-version = 18 : pour l'API Flash Player 11.5
		 * -default-frame-rate 40 : nombre d'images par secondes
		 * 
		 */
		public function Main()
		{
			// attend la fin de l'initialisation de la scène
			this.addEventListener(Event.ADDED_TO_STAGE, onReady, false, 0, true);
		}

		/**
		 * Démarrage de l'application.
		 * définit le nombre d'images par secondes dans les paramètres de compilation
		 * 
		 * 
		 */
		private function onReady(event : Event) : void
		{
			this.removeEventListener(Event.ADDED_TO_STAGE, onReady);
			
			// Init De MonsterDebugger
			MonsterDebugger.initialize(this);
			MonsterDebugger.trace(this, this, "Start Events");
			
			this.buildUI();
		}


		/**
		 * Construit l'affichage de l'application.
		 * 
		 * 
		 */
		private function buildUI() : void
		{			
			var oMultiMedia:SerialElement = createPlaylist();			
			addPlayerVideo(oMultiMedia);
			addPlayerCommand();
		}
		
		
		/**
		 * Création de la playlist avec tous les éléments (vidéos, application swf)
		 * 
		 */
		protected function createPlaylist() : SerialElement
		{
			// création d'une liste de média
			oMediasList = new DefaultMediaFactory();

			// ajout d'une vidéo d'introduction dans la playlist
			var oVideoIntro : MediaElement = oMediasList.createMediaElement(new URLResource(INTRO_VIDEO));

			// création d'une vidéo en streaming, le 2ème paramètre, c'est la durée de lecture en secondes
			var oStreamVideo : StreamingURLResource = new StreamingURLResource(VIDEO_STREAMING, StreamType.RECORDED, 0, 5);
			oStreamVideo.urlIncludesFMSApplicationInstance = true;// pour activer la lecture d'un flux en streaming avec Amazon S3 et CloudFront
			
			// ajout de la vidéo en streaming dans la playlist
			var oVideoStreaming : MediaElement = oMediasList.createMediaElement(oStreamVideo);

			// ajout dans la playlist d'une application swf autonome
			var oSwfApplication : MediaElement = oMediasList.createMediaElement(new URLResource(MAIL_SWF));

			// défini l'évènement que l'application swf va envoyer pour signaler la fin de son exécution et la reprise de la lecture par le player vidéo
			this.addEventListener("validationmail", onReceiveApplicationSwf);

			// création de la playlist avec la liste de tous les éléments exécutés les uns à la suite des autres
			var oSerialElement : SerialElement = new SerialElement();
			oSerialElement.addChild(oVideoIntro);
			oSerialElement.addChild(oVideoStreaming);
			oSerialElement.addChild(oSwfApplication);

			// création d'un objet LayoutMetadata pour définir des paramètres d'affichage communs à la playlist
			var layoutData : LayoutMetadata = new LayoutMetadata();
			layoutData.scaleMode = ScaleMode.LETTERBOX;// Conserve les proportions des vidéos
			layoutData.width = stage.stageWidth;
			layoutData.height = stage.stageHeight - 40;

			oSerialElement.metadata.addValue(LayoutMetadata.LAYOUT_NAMESPACE, layoutData);// ajout des métadata
		
			return oSerialElement;
		}



		/**
		 * Ajoute le lecteur vidéo
		 * 
		 * @param oLogo : le logo à afficher par dessus la vidéo.
		 * 
		 */
		private function addPlayerVideo(oMultiMedia:SerialElement) : void
		{
			OSMFSettings.enableStageVideo = true;// désactive / active la fonctionnalité StageVideo (améliore les performances si active)
			
			// le player
			oMediaPlayer = new MediaPlayer( oMultiMedia );
			oMediaPlayer.addEventListener(TimeEvent.CURRENT_TIME_CHANGE, onCurrentTimeChangeHandler );
			
			
			// container de la vidéo
			oMediasContainer = new MediaContainer();
			oMediasContainer.addMediaElement( oMultiMedia );
			oMediasContainer.addEventListener( MouseEvent.CLICK, onPlayPauseClick, false, 0, true );// clique sur la vidéo pour la mettre en pause / lecture
 			this.addChild( oMediasContainer );			
		}


		/**
		 * Ajoute les boutons du Player
		 * 
		 */
		private function addPlayerCommand() : void
		{
			addMediaPlayerTracks( stage.stageWidth, 10 );// ajoute les barres avance rapide du player		
			
			// durée de la vidéo
			oInfosDuration = this.addText("00:00");
			oInfosDuration.x = 432;
			oInfosDuration.y = 375;
			this.addChild(oInfosDuration);
			
			
			// Création des boutons
			var oBtnPlayPause:DefaultButton = new DefaultButton();
			oBtnPlayPause.getObject().label = "Play / Pause";
			oBtnPlayPause.getObject().x = 10;
			oBtnPlayPause.getObject().y = oInfosDuration.y;
			oBtnPlayPause.getObject().addEventListener ( MouseEvent.CLICK, onPlayPauseClick );// ajout d'un écouteur de l'évènement MouseEvent.CLICK
			this.addChild(oBtnPlayPause);
			
			
			var oBtnMute:DefaultButton = new DefaultButton();
			oBtnMute.getObject().label = "Audio on/off";
			oBtnMute.getObject().x = oBtnPlayPause.getObject().x + oBtnPlayPause.width + 10;
			oBtnMute.getObject().y = oInfosDuration.y;
			oBtnMute.getObject().addEventListener ( MouseEvent.CLICK, onToggleMuteClick );// ajout d'un écouteur de l'évènement MouseEvent.CLICK
			this.addChild(oBtnMute);
		}
		
		/**
 		 * La barre pour naviguer dans la vidéo.
 		 * Création de la barre de progression.
 		 * 
 		 */
		private function addMediaPlayerTracks( nWidth:Number, nHeight:Number ):void
		{
			// barre d'indication de la vidéo
			oTrackVideo = new Sprite();
			oTrackVideo.mouseEnabled = false;// désactive le clique de l'utilisateur
			oTrackVideo.graphics.beginFill( 0x666666, 0.6 );// la couleur - transparence alpha
			oTrackVideo.graphics.drawRect( 0, 0, nWidth, nHeight );
			oTrackVideo.graphics.endFill();
			oTrackVideo.visible = false;// masquée par défaut
			addChild( oTrackVideo );
			
			
			// La barre pour l'avance rapide dans la vidéo.
			oPisteVideo = new Sprite();
			oPisteVideo.addEventListener(MouseEvent.CLICK, onTrackClickPlay, false, 0, true );
			oPisteVideo.graphics.beginFill( 0x696969, 0.8 );
			oPisteVideo.graphics.drawRect( 0, 0, nWidth, nHeight );
			oPisteVideo.graphics.endFill();
			oPisteVideo.visible = false;// masquée par défaut
			addChild( oPisteVideo );
			
			
			// Création de la barre de progression de lecture de la vidéo.
			oProgressPlayVideo = new Sprite();
			oProgressPlayVideo.mouseEnabled = false;// désactive le clique de l'utilisateur
			oProgressPlayVideo.graphics.beginFill( 0xffffff, 0.6 );
			oProgressPlayVideo.graphics.drawRect( 0, 0, nWidth, nHeight );
			oProgressPlayVideo.graphics.endFill();
			oProgressPlayVideo.visible = false;// masquée par défaut
			addChild( oProgressPlayVideo );	
		}
		
		
 
 		/**
 		 * Appel lorsque la vidéo est en lecture, durée écoulée.
 		 * 
 		 */
		private function onCurrentTimeChangeHandler( oEvent:TimeEvent ):void
		{			
			oInfosDuration.text = secondsToMinutes( oEvent.time )+" / "+secondsToMinutes( oMediaPlayer.duration );
			
			var nPourcent:Number = (oEvent.time / oMediaPlayer.duration ); 
			oProgressPlayVideo.scaleX = nPourcent;
			
			// affiche et positionne les barres "avance rapide"
			if( oMediasContainer.height > 0 && !oProgressPlayVideo.visible)
			{
				oProgressPlayVideo.y = oMediasContainer.height;
				oProgressPlayVideo.visible = true;
				
				oPisteVideo.y = oProgressPlayVideo.y;
				oPisteVideo.visible = true;
				
				oTrackVideo.y = oProgressPlayVideo.y;
				oTrackVideo.visible = true;
			}
		}
 	
  						
			
		/**
		 * Appel à chaque appui sur le bouton Play / Pause de la vidéo (ou sur la vidéo)
		 * 
		 */
		private function onPlayPauseClick(event : MouseEvent) : void
		{
			//MonsterDebugger.trace(this,  "onPlayPauseClick");
			// si la player est en lecture --> mise en pause
			if( oMediaPlayer.playing )
				oMediaPlayer.pause();
			else
				oMediaPlayer.play();
		}

		
		/**
		 * Met le son au maximum ou en silence.
		 * 
		 */
		private function onToggleMuteClick(event:MouseEvent):void
		{
			oMediaPlayer.muted = !oMediaPlayer.muted;
		}
		
		
		 /**
 		 * Clique de l'utilisateur sur la barre d'avance rapide.
 		 * 
 		 */
		private function onTrackClickPlay( oEvent:MouseEvent ):void
		{
			MonsterDebugger.trace(this,  "onTrackClickPlay" );
			oMediaPlayer.seek( (oEvent.target.mouseX / oEvent.target.width) * oMediaPlayer.duration );
		}
		

				
		/**
		 * Réception de la réponse du swf appelé dans le player vidéo.
		 * 
		 */
		protected function onReceiveApplicationSwf( oEvent:Event ):void
		{
			MonsterDebugger.trace(this, "réception réponse de l'application swf");

			// création d'une vidéo en streaming, le 2ème paramètre, c'est la durée de lecture en secondes - possibilité de reprendre la lecture de la vidéo juste après l'application swf	
			var oStreamVideo : StreamingURLResource = new StreamingURLResource(VIDEO_STREAMING, StreamType.RECORDED, 5);
			oStreamVideo.urlIncludesFMSApplicationInstance = true;// pour activer la lecture d'un flux en streaming avec Amazon S3 et CloudFront
			
			var oVideoElement:MediaElement = oMediasList.createMediaElement( oStreamVideo  );// ajout de la vidéo en streaming dans la playlist
			
			oMediaPlayer.media = oVideoElement;// ajout de la vidéo au player
			oMediasContainer.addMediaElement( oVideoElement );// ajout d'un élément vidéo dans le container d'affichage du player vidéo	
		}
		
		
		/**
		 * Création d'un champ texte.
		 * 
		 */
		private function addText(sValue:String) : UITextField
		{
			var oTxtField:UITextField = new UITextField();
			oTxtField.text = sValue;
			oTxtField.selectable = false;
			oTxtField.autoSizeLeft();
			oTxtField.alignLeft(); // centre le texte
			oTxtField.changeFormat("bold", true);
			oTxtField.changeFormat("size", 15);// changement de la taille de la police du texte
			oTxtField.changeFormat("font", "Arial");// changement de la taille de la police du texte
			oTxtField.changeFormat("color", 0x00C5DF);// changement de la couleur
			
			return oTxtField;	 
		}		


		/**
		 * Converti des secondes en minutes - 153 secondes en 02:33
		 * 
		 * @param nValue : nombre de secondes
		 * @return : une chaine de caractère au format 00:00
		 */
		private function secondsToMinutes( nSeconds:uint) : String
		{
			var sTime:String;

			// si la durée de la vidéo est supérieur à 1h00
			if ( nSeconds >= 60 * 60 )
			{
				// Format-> H:MM:SS
				sTime = (Math.floor(nSeconds / (60 * 60))) + // Heures 
				":" + ("0" + Math.floor((nSeconds % (60 * 60)) / 60)).substr(-2) + // Minutes 
				":" + ("0" + (nSeconds % 60)).substr(-2);// Secondes
			}
			else
			{
				// Format-> MM:SS
				sTime = ("0" + Math.floor(nSeconds / 60)).substr(-2) + // Minutes 
				":" + ("0" + (nSeconds % 60)).substr(-2);// Secondes
			}
			return sTime;
		}

	}
}
